lorom

;
;   == MEMORY VARIABLES ==
;

!player_1_input = $8F
!oam_stack_ptr = $0590
!screen_fade_delay = $0723
!screen_fade_counter = $0725

;
;   == REGISTERS ==
;

bright = $2100
bg1_sc = $2107
bg2_sc = $2108
bg12_nba = $210B
bg1_offset_y = $210E
vram_low = $2116
m_bg_layers = $212C
dma_enable = $420B

;
;   == FUNCTION LABELS ==
;

; - local -
!FastScreenFadeOut = $90B8
!FastScreenFadeIn = $9100
!AddNintendoLogoSpritemap = $936B

; - global -
!WaitForNMI = $808338
!FinalizeOAM = $80896E
!ClearHighOAM = $808B1A
!BeginDMATransfer = $8091A9

;
;   == WARNING TEXT ==
;

;      ( CONTENT WARNING )
;
;   SUPER METROID DOES FEATURE
;     BRIGHT, FLASHING LIGHTS
;       AND GRAPHIC VISUALS
;
;    METROID IS A PROPERTY OF
;           (Nintendo)
;
;      START - ACKNOWLEDGE
;

; GFX  POINTER: $8E8000
; PAL  POINTER: $8CE1E9
; TMAP POINTER: $8CF3E9

;
;   == CONTENT WARNING TILEMAP ==
;

org $8CF3E9
InfoTilemap: 
    incbin "contentwarning.tmap"

;
;   == CUSTOM, DARKER ZEBES PALETTE ==
;
org $8EE600
ZebesInfoPalette:
    dw $0000, $21B8, $21B4, $21B0, $1D33, $1D30, $14AE, $144A, $1403, $7DFF, $6819, $5413, $340A, $2004, $1403, $0000

;
;   == SPLASH SCREEN REFACTOR ==
;

org $8B92E2
START_FADE:
    JSL InitializeInfoScreen
    LDA #$0002 : STA !screen_fade_counter

LOOP_FADE_IN:
    JSR ClearOAMLoadNintendoLogo
    JSR !FastScreenFadeIn
    BCS LOOP_NINTENDO_LOGO
    JSR FinalizeOAMAndWaitForNMI
    BRA LOOP_FADE_IN

LOOP_NINTENDO_LOGO:
    JSR ClearOAMLoadNintendoLogo

    LDA !player_1_input ; Check player input
    BIT #$1000 ; Is START pressed?
    BNE NL_END ; If so, end LOOP_NINTENDO_LOGO
    ;            If not, continue loop
    JSR FinalizeOAMAndWaitForNMI
    BRA LOOP_NINTENDO_LOGO

NL_END:
    LDA #$0001 : STA !screen_fade_counter

LOOP_FADE_OUT:
    JSR ClearOAMLoadNintendoLogo
    JSR !FastScreenFadeOut
    BCS FO_END
    JSR FinalizeOAMAndWaitForNMI
    BRA LOOP_FADE_OUT

FO_END:
    SEP #$30
    LDA #$8F : STA $51 : STA bright
    REP #$30
    PLB : PLP
    RTL

ClearOAMLoadNintendoLogo:
    JSL !ClearHighOAM : STZ !oam_stack_ptr
    JSR !AddNintendoLogoSpritemap
    RTS

FinalizeOAMAndWaitForNMI:
    JSL !FinalizeOAM
    JSL !WaitForNMI
    RTS

org $8B937E
NudgeNintendoLogoDown:
    LDA #$0094

;
;   == WARNING SCREEN CODE ==
;

org $8BF780
InitializeInfoScreen:
    SEP #$30

    ; Enable BG1 + BG2 + Sprites
    LDA #%00010011 : STA $69 : STA m_bg_layers

    ; Set BG1 + BG2 tile addresses
    LDA #$33 : STA $5D : STA bg12_nba ; Point to VRAM address $6000

    REP #$30

LoadZebesPalette:
    LDA ZebesInfoPalette,x
    STA $7EC080,x ; Store into CGRAM address $0080
    INX
    CPX #$0020
    BMI LoadZebesPalette

LoadTextTiles:
    LDA #$3000 : STA vram_low ; Start storing @ tile address $6000
    SEP #$30

    JSL !BeginDMATransfer
    db $01, $01, $18
    dl $8E8000 ; Text tiles location
    dw $1600 ; Amount of tiles to store into memory
    LDA #$02 : STA dma_enable ; Send info to DMA

LoadZebesTiles:
    REP #$30
    LDA #$5000 : STA vram_low ; Start storing @ tile address $A000
    SEP #$30

    JSL !BeginDMATransfer
    db $01, $01, $18
    dl $8EC000 ; Zebes planet tiles location
    dw $1A00 ; Amount of tiles to store into memory
    LDA #$02 : STA dma_enable ; Send info to DMA

LoadInfoTilemap:
    REP #$30
    LDA #$1400 : STA vram_low ; Start storing @ tile address $2800
    SEP #$30
    
    JSL !BeginDMATransfer
    db $01, $01, $18
    dl InfoTilemap
    dw $800 ; Amount of tilemap info to store into memory
    LDA #$02 : STA dma_enable ; Send information to DMA

LoadZebesTilemap:
    REP #$30
    LDA #$2000 : STA vram_low ; Start storing @ address $4000
    SEP #$30
    
    JSL !BeginDMATransfer
    db $01, $01, $18
    dl $8EDC00 ; Zebes tilemap location
    dw $800 ; Amount of tilemap info to store into memory
    LDA #$02 : STA dma_enable ; Send information to DMA

SetTilemapAddresses:
    LDA #$14 : STA $58 : STA bg1_sc
    LDA #$20 : STA $59 : STA bg2_sc

NudgeBG1Up: ; optional
    ; 
    ;       I was slightly off in my tilemap work
    ;       
    ;       don't make the same mistake I did
    ;           unless you wanna waste a few cycles
    ;
    ;       either way the performance difference is barely noticeable
    ;           but still, be smarter than me.
    ;
    LDA #$0E : STA $B3 : STA bg1_offset_y

FinishInitialization:
    REP #$30
    RTL
